library(loadeR)
library(transformeR)
library(ggplot2)
library(RColorBrewer)
library(maps)
library(gridExtra)
library(Kendall)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("~/Documents/ETCCDI_eobs/")
OUTPUTS <- "OUTPUTS/"

calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

#################################################################################
#Load NAO data
data <- read.csv("~/Downloads/cpc.ncep.noaa.gov_products_precip_CWlink_pna_norm.nao.monthly.b5001.current.ascii.table.csv",colClasses = "numeric")
data2 <- as.matrix(data)
serie <- as.numeric(t(data2[1:72,2:13])) #monthly

NAO <- loadGridData(dataset = "../eobs_tasmax_spain.nc",lonLim = c(-1,1),latLim = c(35,36),var="tasmax",aggr.m = "mean")
NAO <- aggregateGrid(NAO,aggr.spatial = list(FUN="mean"))

for(i in 1:length(serie)){
  NAO$Data[i] <- serie[i]
}

NAO.winter <- aggregateGrid(NAO,aggr.s = list(FUN=list("sum",na.rm=T),season=c(12,1,2)))
NAO.autumn <- aggregateGrid(NAO,aggr.s = list(FUN=list("sum",na.rm=T),season=c(3,4,5)))
NAO.summer <- aggregateGrid(NAO,aggr.s = list(FUN=list("sum",na.rm=T),season=c(6,7,8)))
NAO.spring <- aggregateGrid(NAO,aggr.s = list(FUN=list("sum",na.rm=T),season=c(9,10,11)))

datitos <- data.frame(NAO=c(NAO.winter$Data,NAO.autumn$Data,NAO.summer$Data,NAO.spring$Data),
                      Season=c(rep("Winter",71),rep(c("Autumn","Summer","Spring"),each=72)),
                      Years=c(1951:2021,1950:2021,1950:2021,1950:2021))
datitos$Season <- factor(datitos$Season,levels=c("Winter","Autumn","Summer","Spring"))
p <- ggplot(data=datitos,aes(x=Years,y=NAO)) +geom_line(col="blue") + facet_wrap(~Season,nrow=4)  +theme_bw()+
  geom_hline(yintercept =0,col) 

################################################################################
#Load CPs information
load("../WardCluster_Dendrogram_D2_50km.Rdata")
k=7
vector_cluster <- cutree(ward_clustering,k)[which(format(calendario,"%Y")%in%c(1950:2021))]
cp <- loadGridData(dataset = "../eobs_tasmax_spain.nc",lonLim = c(-1,1),latLim = c(35,36),var="tasmax")
cp <- aggregateGrid(cp,aggr.spatial = list(FUN="mean"))
for(l in 1:length(cp$Data)){
  cp$Data[l] <- vector_cluster[l]
}

################################################################################
value.winter <- NA ; value.autumn <- NA ; value.summer <- NA ; value.spring <- NA
sign.winter <- NA ; sign.autumn <- NA ; sign.summer <- NA ; sign.spring <- NA

for(l in 1:length(unique(vector_cluster))){
  v <- unique(vector_cluster)[l]
  cp2 <- cp
  cp2$Data[which(cp2$Data!=v)] <- 0
  cp2$Data[which(cp2$Data==v)] <- 1
  
  index.cp.winter <- aggregateGrid(cp2,aggr.s = list(FUN=list("sum",na.rm=T),season=c(12,1,2)))
  index.cp.autumn <- aggregateGrid(cp2,aggr.s = list(FUN=list("sum",na.rm=T),season=c(3,4,5)))
  index.cp.summer <- aggregateGrid(cp2,aggr.s = list(FUN=list("sum",na.rm=T),season=c(6,7,8)))
  index.cp.spring <- aggregateGrid(cp2,aggr.s = list(FUN=list("sum",na.rm=T),season=c(9,10,11)))
  
  #Removing trend in CP frequency and NAO index (if any)
  index.cp <- index.cp.winter
  trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
  index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
  trendfreq  <- lm(NAO.winter$Data~c(1:length(unique(getYearsAsINDEX(NAO.winter)))))[["coefficients"]][2]
  NAO.winter$Data <- NAO.winter$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(NAO.winter))))
  value.winter[l] <- as.numeric(Kendall(x=index.cp$Data,y=NAO.winter$Data)$tau)
  sign.winter[l] <- as.numeric(Kendall(x=index.cp$Data,y=NAO.winter$Data)$sl)
  
  index.cp <- index.cp.autumn
  trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
  index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
  trendfreq  <- lm(NAO.autumn$Data~c(1:length(unique(getYearsAsINDEX(NAO.autumn)))))[["coefficients"]][2]
  NAO.autumn$Data <- NAO.autumn$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(NAO.autumn))))
  value.autumn[l] <- as.numeric(Kendall(x=index.cp$Data,y=NAO.autumn$Data)$tau)
  sign.autumn[l] <- as.numeric(Kendall(x=index.cp$Data,y=NAO.autumn$Data)$sl)
  
  index.cp <- index.cp.summer
  trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
  index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
  trendfreq  <- lm(NAO.summer$Data~c(1:length(unique(getYearsAsINDEX(NAO.summer)))))[["coefficients"]][2]
  NAO.summer$Data <- NAO.summer$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(NAO.summer))))
  value.summer[l] <- as.numeric(Kendall(x=index.cp$Data,y=NAO.summer$Data)$tau)
  sign.summer[l] <- as.numeric(Kendall(x=index.cp$Data,y=NAO.summer$Data)$sl)
  
  index.cp <- index.cp.spring
  trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
  index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
  trendfreq  <- lm(NAO.spring$Data~c(1:length(unique(getYearsAsINDEX(NAO.spring)))))[["coefficients"]][2]
  NAO.spring$Data <- NAO.spring$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(NAO.spring))))
  value.spring[l] <- as.numeric(Kendall(x=index.cp$Data,y=NAO.spring$Data)$tau)
  sign.spring[l] <- as.numeric(Kendall(x=index.cp$Data,y=NAO.spring$Data)$sl)
}

datitos.cor <- data.frame(Tau=c(value.winter,value.autumn,value.summer,value.spring),
                          Sign=c(sign.winter,sign.autumn,sign.summer,sign.spring),
                          CP=rep(1:k,4),
                          Season=rep(c("Winter","Autumn","Summer","Spring"),each=k))
datitos.cor$Season <- factor(datitos.cor$Season,levels=c("Winter","Autumn","Summer","Spring"))

h <- ggplot(data=datitos.cor,aes(x=Season,y=CP,fill=Tau)) + geom_tile() + theme_bw() + 
  scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-0.75,0.75),
                        aesthetics = "fill",na.value = "white") + coord_cartesian(expand=F)+
  geom_text(data=subset(datitos.cor,Sign<=0.05),aes(label=round(Tau,digits=2)),fontface="bold")+
  geom_text(data=subset(datitos.cor,Sign>0.05),aes(label=round(Tau,digits=2))) + scale_y_continuous(n.breaks=7)


png(filename = "../Cor_CP_NAO_D2.png",res=300,width=4000,height=1500)
print(grid.arrange(p,g,h,nrow=1))
dev.off()
